/******************************************************************************
* Copyright (c) 2005,2006, Novell, Inc.  All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*	- Redistributions of source code must retain the above copyright notice,
*	this list of conditions and the following disclaimer.
*
*	- Redistributions in binary form must reproduce the above copyright notice,
*  	this list of conditions and the following disclaimer in the documentation
*	and/or other materials provided with the distribution.
*
*	- Neither the name of Novell nor the names of its contributors may be
*	used to endorse or promote products derived from this software without
*	specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/

#ifndef SMASHDEV_TYPES_HPP_GUARD
#define SMASHDEV_TYPES_HPP_GUARD

#include <openwbem/OW_String.hpp>
#include <openwbem/OW_CIMInstance.hpp>

namespace OMCSmash
{

enum EDeviceType
{
	// Other
	E_DEV_OTHER = 1					// IPMI
	// Unspecified
	, E_DEV_UNKNOWN = 2				// IPMI
	// Processor
	, E_DEV_PROCESSOR = 3			// IPMI
	// Disk or drive bay
	, E_DEV_DRIVE_BAY = 4			// IPMI
	// Peripheral bay
	, E_DEV_PERIPHERAL_BAY = 5		// IPMI
	// System management module
	, E_DEV_SYS_MGMT = 6			// IPMI
	// System board (main system board, may also be a processor board and/or
	// internal expansion board)
	, E_DEV_SYS_BOARD = 7			// IPMI
	// Memory module (board holding meory devices)
	, E_DEV_MEM_MODULE = 8			// IPMI
	// Processor module (holds processors. Used when processor are not 
	// mounted on the system board)
	, E_DEV_PROC_MODULE = 9			// IPMI
	// Power supply. Used  for the main power supply (supplies) for the
	// system.
	, E_DEV_POWER_SUPPLY = 10		// IPMI
	// Add in card
	, E_DEV_ADD_IN_CARD = 11		// IPMI
	// Front panel board (control panel)
	, E_DEV_FRONT_PANEL = 12 		// IPMI
	// Back panel board
	, E_DEV_BACK_PANEL = 13 		// IPMI
	// Power system board
	, E_DEV_PWR_SYS_BOARD = 14 		// IPMI
	// Drive backplane
	, E_DEV_DRIVE_BACKPLANE = 15 	// IPMI
	// System internal expansion board (contains expansion slots)
	, E_DEV_SYS_INT_EXP_BOARD = 16 	// IPMI
	// Other system board (part of board set)
	, E_DEV_OTHER_SYS_BOARD = 17 	// IPMI
	// Processor board (holds one or more processors - includes boards that hold
	// SECC modules)
	, E_DEV_PROC_BOARD = 18			// IPMI
	// Power unit/power domain - This is represents the grouping of 
	// power supplies.
	, E_DEV_PWR_UNIT = 19			// IPMI 
	// Power module / dc-to-dc converter. Used for internal converters
	, E_DEV_PWR_MODULE = 20			// IPMI 
	// Power management / power distribution board
	, E_DEV_PWR_MGMT = 21			// IPMI 
	// Chassis back panel board
	, E_DEV_CHAS_BACK_PANEL_BRD = 22	// IPMI 
	// System chassis
	, E_DEV_SYS_CHASSIS = 23			// IPMI 
	// Sub-chassis
	, E_DEV_SUB_CHASSIS = 24			// IPMI 
	// Other chassis board
	, E_DEV_OTHER_CHASSIS_BRD = 25	// IPMI 
	// Disk drive bay
	, E_DEV_DISK_DRIVE_BAY = 26		// IPMI 
	// Peripheral bay
	, E_DEV_PERIPHERAL_BAY2 = 27	// IPMI 
	// Device bay
	, E_DEV_DEVICE_BAY = 28			// IPMI 
	// Fan / cooling device
	, E_DEV_FAN = 29 				// IPMI
	// Cooling unit - Can be used for grouping fans or other
	// cooling devices.
	, E_DEV_COOLING_UNIT =  30 		// IPMI
	// Cable / interconnect
	, E_DEV_CABLE_INTERCONNECT = 31	// IPMI
	// Memory device - Used for replaceable memory devices. e.g. DIMM/SIMM.
	// It is recommended this NOT be used for individual non-replaceable memory
	// devices. Monitoring and error reporting should be associated with the
	// FRU [e.g memory card] holding the memory.
	, E_DEV_MEMORY_DEVICE = 32		// IPMI 
	// System management software
	, E_DEV_SYS_MGMT_SOFTWARE = 33	// IPMI
	// System firmware (e.g. BIOS)
	, E_DEV_SYS_FIRMWARE = 34 		// IPMI
	// Operating system
	, E_DEV_OS = 35 				// IPMI
	// System bus
	, E_DEV_SYS_BUS = 36 			// IPMI
	// Entity association - Not used
	, E_DEV_ENTITY_ASSOC = 37 		// IPMI
	// Remote (out of band) management communication device.
	, E_DEV_REMOTE_MGMT_DEVICE = 38 	// IPMI
	// External device. This ID can be used to identify the environment
	// outside the system chassis. For example, a system may have a 
	// temperature sensor that monitors the temperature “outside the box”.
	// Such a temperature sensor can be associated with an External
	// Environment entity. This value will typically be used as a single
	// instance physical entity. However, the Entity Instance value can be
	// used to denote a difference in regions of the external environment.
	// For example, the region around the front of a chassis may be considered
	// to be different from the region around the back, in which case it would
	// be reasonable to have two different instances of the External
	// Environment entity.
	, E_DEV_EXTERNAL_ENV = 39 		// IPMI
	// Battery
	, E_DEV_BATTERY = 40 			// IPMI
	// Processing blade (a blade module that contains processor, memory, and
	// I/O connection that enable it to operatew as a processing entity)
	, E_DEV_PROC_BLADE = 41			// IPMI
	// Connectivity switch (a blade module that provides the fabric or network
	// connection for one or more processing blades or modules)
	, E_DEV_CONN_SWITCH = 42 		// IPMI
	// Processor / memory module (processor and memory together on a module
	, E_DEV_PROCMEM_MOD = 43 		// IPMI
	// I/O module (a module that contains the main elements of an I/O
	// interface)
	, E_DEV_IO_MOD = 44 			// IPMI
	// Processor/IO moddule (a module that contains the main element of an 
	// I/O interface
	, E_DEV_PROC_IO_MOD = 45 		// IPMI
	// Management controller firmware (represents firmware or software 
	// running on a management controller
	, E_DEV_MGMT_CONT_FIRMWARE = 46 // IPMI
	// IPMI Channel - Allows for associating sensors with IPMI communication
	// channels.
	, E_DEV_IPMI_CHANNEL = 47 		// IPMI
	// PCI bus
	, E_DEV_PCI_BUS = 48 			// IPMI
	// PCI express bus
	, E_DEV_PCI_EXPRESS_BUS = 49 	// IPMI
	// SCSI Bus (parallel)
	, E_DEV_SCSI_BUS = 50 			// IPMI
	// SATA / SAS bus
	, E_DEV_SATA_BUS = 51 			// IPMI
	// Processor / front-side bus
	, E_DEV_PROC_FRONT_SIDE_BUS = 52 // IPMI
	// Real time clock (RTC)
	, E_DEV_RTC = 53				// IPMI 
	// System firmware (e.g. BIOS/EFI) - don't know what the difference is
	// between this and 34
	, E_DEV_SYS_FIRMWARE2 = 54,		// IPMI 

	/*
	 * End of IPMI Devices
	 *
	 * Add other devices here
	 */
	E_DEV_KEYBOARD                        = 1000,       // Keyboard
	E_DEV_MOUSE                           = 1001,       // Mouse
	E_DEV_JOYSTICK                        = 1002,       // Joystick
	E_DEV_PRINTER                         = 1003,       // Printer
	E_DEV_SCANNER                         = 1004,       // Scanner
	E_DEV_CHIPCARD                        = 1005,       // Chipcard
	E_DEV_MONITOR                         = 1006,       // Monitor
	E_DEV_TV                              = 1007,       // TV
	E_DEV_DISPLAY                         = 1008,       // Display
	E_DEV_CAMERA                          = 1009,       // Camera
	E_DEV_SOUND                           = 1010,       // Sound
	E_DEV_STORAGE_CTRL                    = 1011,       // Storage control
	E_DEV_NETWORK_CTRL                    = 1012,       // Network control
	E_DEV_ISDN                            = 1013,       // ISDN
	E_DEV_MODEM                           = 1014,       // Modem
	E_DEV_NETWORK                         = 1015,       // Network - TODO Correlation?
	E_DEV_DISK                            = 1016,       // Disk
	E_DEV_CDROM                           = 1017,       // CDROM
	E_DEV_FLOPPY                          = 1018,       // Floppy drive
	E_DEV_USB_CTRL                        = 1019,       // USB Control
	E_DEV_USB                             = 1020,       // USB
	E_DEV_PCI                             = 1021,       // PCI - TODO Correlation?
	E_DEV_HUB                             = 1022,       // HUB - TODO Correlation?
	E_DEV_SCSI                            = 1023,       // SCSI - TODO Correlation?
	E_DEV_IDE                             = 1024,       // IDE - TODO Correlation?
	E_DEV_DVB                             = 1025,       // DVB	?
	E_DEV_PCMCIA                          = 1026,       // PCMCIA
	E_DEV_PCMCIA_CTRL                     = 1027,       // PCMCIA Control
	E_DEV_IEEE1394                        = 1028,       // IEEE1394 
	E_DEV_IEEE1394_CTRL                   = 1029,       // IEEE1394 Control
	E_DEV_HOTPLUG                         = 1030,       // Hot plug
	E_DEV_HOTPLUG_CTRL                    = 1031,       // Hot plug control
	E_DEV_ZIP                             = 1032,       // Zip
	E_DEV_PPPOE                           = 1033,       // PPPOE
	E_DEV_WLAN                            = 1034,       // WLAN
	E_DEV_REDASD                          = 1035,       // REDASD
	E_DEV_DSL                             = 1036,       // DSL
	E_DEV_BLOCK                           = 1037,       // Block device TODO: disk/floppy/cdrom?
	E_DEV_TAPE                            = 1038,       // Tape
	E_DEV_VBE                             = 1039,       // VBE
	E_DEV_BLUETOOTH                       = 1040,       // Bluetooth
	
	/*
	 * Normal system memory is in the CIM_MEMORY class. However,
	 * cache memory is also considered CIM_MEMORY, so we will add
	 * cache memory to help us differentiate between normal memory
	 * and cache memory.
	 */
	E_DEV_L1_CACHE                        = 1041,       // Level-1 Cache
	E_DEV_L2_CACHE                        = 1042,       // Level-2 Cache
	E_DEV_L3_CACHE                        = 1043,       // Level-3 Cache

	/*
	 * Connectors
	 */
	E_DEV_CONNECTOR                       = 1044,       // Physical connector

	/*
	 * Slots
	 *
	 * Since we have no way to identity the slots, each one must have a unique device number.
	 */
	E_DEV_SLOT                            = 1045,       // Physical slot
	E_DEV_SLOT_OTHER                      = 1046,       // other slot
	E_DEV_SLOT_UNKNOWN                    = 1047,       // unknown slot
	E_DEV_SLOT_ISA                        = 1048,       // ISA slot
	E_DEV_SLOT_MCA                        = 1049,       // MCA (micro channel adaptor) slot
	E_DEV_SLOT_EISA                       = 1050,       // EISA slot
	E_DEV_SLOT_PCI                        = 1051,       // PCI slot
	E_DEV_SLOT_PCMCIA                     = 1052,       // PCMCIA or PC Card slot
	E_DEV_SLOT_VL_VESA                    = 1046,       // VL-VESA slot
	E_DEV_SLOT_PROPRIETARY                = 1047,       // Propietary slot
	E_DEV_SLOT_PROCESSOR_CARD             = 1048,       // Processor card slot
	E_DEV_SLOT_PROPRIETARY_MEMORY_CARD    = 1049,       // Propietary memory card slot
	E_DEV_SLOT_IO_RISER_CARD_SLOT         = 1050,       // I/O riser card slot
	E_DEV_SLOT_NUBUS                      = 1051,       // NuBus slot
	E_DEV_SLOT_PCI_66MHZ                  = 1052,       // PCI (66 MHz capable) slot
	E_DEV_SLOT_AGP                        = 1053,       // AGP slot
	E_DEV_SLOT_AGP_2X                     = 1054,       // AGP (2X) slot
	E_DEV_SLOT_AGP_4X                     = 1055,       // AGP (4X) slot
	E_DEV_SLOT_PCI_X                      = 1056,       // PCI (4X) slot
	E_DEV_SLOT_AGP_8X                     = 1057,       // AGP (8X) slot
	E_DEV_SLOT_PC_98_C20                  = 1058,       // PC-98/C20 slot
	E_DEV_SLOT_PC_98_C24                  = 1059,       // PC-98/C24 slot
	E_DEV_SLOT_PC_98_E                    = 1060,       // PC-98/E slot
	E_DEV_SLOT_PC_98_LOCAL_BUS            = 1061,       // PC-98/Local Bus slot
	E_DEV_SLOT_PC_98_CARD                 = 1062,       // PC-98/Card slot
	E_DEV_SLOT_PCI_EXPRESS                = 1063,       // PCI Express slot

	// Logical Devices
	E_DEV_LOGICAL_MEMORY                  = 2000,
};


/**
 * Create a device key from a given device number and sequence number.
 * @param deviceNumber The device number part of the key. This will be an 
 *		EDeviceType value.
 * @param sequenceNumber The sequence number part of the key.
 * @return The device key as a String object.
 */
OpenWBEM::String createDeviceKey(
	int deviceNumber,
	int sequenceNumber);

OpenWBEM::String createDeviceKey(
	int   deviceNumber,
	char *sequenceNumber);

/**
 * Parse a String representation of a device key.
 * @param strKey String representation of the key to parse.
 * @param deviceNumberOut Will contain the device number on successful
 *		return.
 * @param sequenceNumberOut Will contain the sequence number on successful
 *		return.
 * @return true if strKey was parsed successfully. Otherwise false.
 */
bool parseDeviceKey(
	const OpenWBEM::String& strKey,
	int& deviceNumberOut, 
	int& sequenceNumberOut);

/**
 * Get a value for a given variable name out of the 'OtherIdentifyingInfo'
 * property.
 * @param varName The variable name to retrieve the value for.
 * @param ci The CIMInstance to get the 'OtherIdentifyingInfo' property from.
 * @return The value of the given variable from the 'OtherIdentifyingInfo'
 *      property on success. Otherwise an empty string.
 */
OpenWBEM::String getOIIValue(
    const OpenWBEM::String& varName, 
    const OpenWBEM::CIMInstance& ci);


/**
 * Given a physical element class name and a tag, return the appropriate 
 * CIMObjectPath
 * @param physicalElementClassName
 * @param tag
 * @param ns
 * @return CIMObjectPath: the object path to the object
 */
OpenWBEM::CIMObjectPath createPhysicalElementObjectPath(
    const OpenWBEM::String &physicalElementClassName,
    const OpenWBEM::String &tag,
    const OpenWBEM::String &ns);

OpenWBEM::CIMObjectPath createLogicalDeviceObjectPath(
    const OpenWBEM::String &logicalDeviceClassName,
    const OpenWBEM::String &deviceId,
    const OpenWBEM::String &ns);

/**
 * Convert a IPMI entity ID to a device ID that will be used in a
 * correlatable ID.
 * @param entityId The IPMI entity ID to convert.
 * @return The correlatable device ID on success. Otherwise -1.
 */
int ipmiEntityId2DeviceId(
    int entityId);

/**
 * Convert a device ID that is used in a correlatable ID to an IPMI entity ID.
 * @param deviced The device ID to convert.
 * @return The IPMI entity ID on success. Otherwise -1.
 */
int deviceId2IPMIEntityId(
    int deviceId);

/**
 * Determine the Physical class name for the given IPMI entity id.
 * @param entityId The entity id to determine the class names for.
 * @param physicalClassName This is an output parameter. It will contain the
 *      physical class name for the given entity if it is found. Note: Not
 *      every entity has a physical class name.
 * @return true if the entity was found. Otherwise false.
 */
bool ipmiEntityId2Physical(
	int entityId,
	OpenWBEM::String& physicalClassName);

/**
 * Determine the logical class names for the given IPMI entity ID.
 * @param entityId The entity id to determine the class names for.
 * @param logicalClassName This is an output parameter. It will contain the
 *      logical class name for the given entity if it is found. Note: Not
 *      every entity has a logical class name.
 * @return true if the entity was found. Otherwise false.
 */ 
bool ipmiEntityId2Logical(
	int entityId,
	OpenWBEM::String& logicalClassName);

/**
 * Determine the logical and physical class names for the given IPMI entity ID.
 * @param entityId The entity id to determine the class names for.
 * @param physicalClassName This is an output parameter. It will contain the
 *      physical class name for the given entity if it is found. Note: Not
 *      every entity has a physical class name.
 * @param logicalClassName This is an output parameter. It will contain the
 *      logical class name for the given entity if it is found. Note: Not
 *      every entity has a logical class name.
 * @return true if the entity was found. Otherwise false.
 */ 
bool ipmiEntityId2LogicalPhysical(
    int entityId,
    OpenWBEM::String& physicalClassName,
    OpenWBEM::String& logicalClassName);

/**
 * Determine the logical and physical class names for the given device id.
 * @param deviceId The device id to determine the class names for.
 * @param physicalClassName This is an output parameter. It will contain the
 *      physical class name for the given device id if it is found. Note: Not
 *      every device has a physical class name.
 * @param logicalClassName This is an output parameter. It will contain the
 *      logical class name for the given device id if it is found. Note: Not
 *      every device has a logical class name.
 * @return true if the device id was found. Otherwise false.
 */ 
bool deviceId2LogicalPhysical(
    int deviceId,
    OpenWBEM::String& physicalClassName,
    OpenWBEM::String& logicalClassName);

/**
 * Determine the logical or physical class name for the given device id.
 * @param deviceId The device id to determine the class names for.
 * @return The logical or physical class name that corresponds to deviceId
 *      if found. Otherwise an empty string.
 */ 
OpenWBEM::String deviceId2LogicalClass(int deviceId);
OpenWBEM::String deviceId2PhysicalClass(int deviceId);

/**
 * Determine the physical class name for the given deviceId string.
 * The device id string is used as the DeviceID key on logical devices and
 * the Tag key on physical elements.
 * @param devidKey The string value of the DeviceID or Tag key properties from
 * 		a LogicalDevice or PhysicalElement respectively.
 * @return The name of the physical class that corresponds to the device key on
 * success. An empty string if the determination cannot be made.
 */
OpenWBEM::String deviceIdKey2PhysicalClass(const OpenWBEM::String& devidKey);

/**
 * Determine the logical class name for the given deviceId string.
 * The device id string is used as the DeviceID key on logical devices and
 * the Tag key on physical elements.
 * @param devidKey The string value of the DeviceID or Tag key properties from
 * 		a LogicalDevice or PhysicalElement respectively.
 * @return The name of the logical class that corresponds to the device key on
 * success. An empty string if the determination cannot be made.
 */
OpenWBEM::String deviceIdKey2LogicalClass(const OpenWBEM::String& devidKey);

/**
 * Determine if a given class name is a logical class name for hardware
 * instrumented by omc-smash
 * @param logicalClassName The name of the class to check.
 * @return true if logicalClassName is the name of a logical class 
 * 		instrumented by omc-smash
 */
bool isSmashLogicalClass(
    const OpenWBEM::String& logicalClassName);

/**
 * Determine if a given class name is a physical class name for an hardware
 * instrumented by omc-smash
 * @param physicalClassName The name of the class to check.
 * @return true if physicalClassName is the name of a physical class 
 * 		instrumented by omc-smash
 */
bool isSmashPhysicalClass(
    const OpenWBEM::String& physicalClassName);

/**
 * Determine if the logical class has a SystemDevice association.
 * @param className The logical class name to determine the SystemDevice
 *      relationship for.
 * @param deviceId  The deviceId to determine whether a SystemDevice association
 *      should exist
 * @return true if there is a SystemDevice association. Otherwise false.
 */
bool logicalIsSystemDevice(const OpenWBEM::String& className);
bool logicalIsSystemDevice(int deviceId);

/**
 * Determine if IPMI is available on the system.
 * @param errMsg An output parameter that will contain an error
 *		message on return. This will be an empty string if no
 *		error occurred during the determination.
 * @return true if the current system is IPMI enabled. Otherwise false.
 */
bool ipmiAvailable(OpenWBEM::String& errMsg);


}	// End of namespace OMCSmash



//
// OtherIdentifyingInfo keys
// 
#define OIIKEY_MONITORED_IPMIDEVICE     "[monitoredipmidevice]"
#define OIIKEY_MONITORED_DEVICE         "[monitoreddevice]"
#define OIIKEY_IPMI_ENTITY_INFO         "[ipmientityinfo]"

#endif	// SMASHDEV_TYPES_HPP_GUARD
